# Generated from ChordLabel.g4 by ANTLR 4.7
# encoding: utf-8
from antlr4 import *
from io import StringIO
from typing.io import TextIO
import sys

def serializedATN():
    with StringIO() as buf:
        buf.write("\3\u608b\ua72a\u8133\ub9ed\u417c\u3be7\u7786\u5964\3(")
        buf.write("V\4\2\t\2\4\3\t\3\4\4\t\4\4\5\t\5\4\6\t\6\4\7\t\7\4\b")
        buf.write("\t\b\4\t\t\t\4\n\t\n\4\13\t\13\4\f\t\f\3\2\3\2\3\2\3\2")
        buf.write("\3\2\5\2\36\n\2\5\2 \n\2\5\2\"\n\2\3\2\5\2%\n\2\3\2\5")
        buf.write("\2(\n\2\3\3\3\3\7\3,\n\3\f\3\16\3/\13\3\3\4\3\4\3\5\3")
        buf.write("\5\3\6\3\6\3\6\3\6\7\69\n\6\f\6\16\6<\13\6\3\6\3\6\3\7")
        buf.write("\5\7A\n\7\3\7\3\7\3\b\3\b\3\t\7\tH\n\t\f\t\16\tK\13\t")
        buf.write("\3\t\3\t\3\n\3\n\3\13\3\13\3\13\3\f\3\f\3\f\2\2\r\2\4")
        buf.write("\6\b\n\f\16\20\22\24\26\2\5\3\2\30\31\3\2\32&\7\2\b\26")
        buf.write("  \"\"$$&&\2S\2\'\3\2\2\2\4)\3\2\2\2\6\60\3\2\2\2\b\62")
        buf.write("\3\2\2\2\n\64\3\2\2\2\f@\3\2\2\2\16D\3\2\2\2\20I\3\2\2")
        buf.write("\2\22N\3\2\2\2\24P\3\2\2\2\26S\3\2\2\2\30!\5\4\3\2\31")
        buf.write("\37\7(\2\2\32 \5\n\6\2\33\35\5\26\f\2\34\36\5\n\6\2\35")
        buf.write("\34\3\2\2\2\35\36\3\2\2\2\36 \3\2\2\2\37\32\3\2\2\2\37")
        buf.write("\33\3\2\2\2 \"\3\2\2\2!\31\3\2\2\2!\"\3\2\2\2\"$\3\2\2")
        buf.write("\2#%\5\24\13\2$#\3\2\2\2$%\3\2\2\2%(\3\2\2\2&(\7\'\2\2")
        buf.write("\'\30\3\2\2\2\'&\3\2\2\2(\3\3\2\2\2)-\5\6\4\2*,\5\b\5")
        buf.write("\2+*\3\2\2\2,/\3\2\2\2-+\3\2\2\2-.\3\2\2\2.\5\3\2\2\2")
        buf.write("/-\3\2\2\2\60\61\7\27\2\2\61\7\3\2\2\2\62\63\t\2\2\2\63")
        buf.write("\t\3\2\2\2\64\65\7\3\2\2\65:\5\f\7\2\66\67\7\4\2\2\67")
        buf.write("9\5\f\7\28\66\3\2\2\29<\3\2\2\2:8\3\2\2\2:;\3\2\2\2;=")
        buf.write("\3\2\2\2<:\3\2\2\2=>\7\5\2\2>\13\3\2\2\2?A\5\16\b\2@?")
        buf.write("\3\2\2\2@A\3\2\2\2AB\3\2\2\2BC\5\20\t\2C\r\3\2\2\2DE\7")
        buf.write("\6\2\2E\17\3\2\2\2FH\5\b\5\2GF\3\2\2\2HK\3\2\2\2IG\3\2")
        buf.write("\2\2IJ\3\2\2\2JL\3\2\2\2KI\3\2\2\2LM\5\22\n\2M\21\3\2")
        buf.write("\2\2NO\t\3\2\2O\23\3\2\2\2PQ\7\7\2\2QR\5\20\t\2R\25\3")
        buf.write("\2\2\2ST\t\4\2\2T\27\3\2\2\2\13\35\37!$\'-:@I")
        return buf.getvalue()


class ChordLabelParser ( Parser ):

    grammarFileName = "ChordLabel.g4"

    atn = ATNDeserializer().deserialize(serializedATN())

    decisionsToDFA = [ DFA(ds, i) for i, ds in enumerate(atn.decisionToState) ]

    sharedContextCache = PredictionContextCache()

    literalNames = [ "<INVALID>", "'('", "','", "')'", "'*'", "'/'", "'maj'", 
                     "'min'", "'dim'", "'aug'", "'maj7'", "'min7'", "'dim7'", 
                     "'hdim7'", "'minmaj7'", "'maj6'", "'min6'", "'maj9'", 
                     "'min9'", "'sus4'", "'sus2'", "<INVALID>", "'b'", "'#'", 
                     "'1'", "'2'", "'3'", "'4'", "'5'", "'6'", "'7'", "'8'", 
                     "'9'", "'10'", "'11'", "'12'", "'13'", "'N'", "':'" ]

    symbolicNames = [ "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "NATURAL", "FLAT", "SHARP", "I_1", "I_2", 
                      "I_3", "I_4", "I_5", "I_6", "I_7", "I_8", "I_9", "I_10", 
                      "I_11", "I_12", "I_13", "NO_CHORD", "SEPARATOR" ]

    RULE_chord = 0
    RULE_root = 1
    RULE_natural = 2
    RULE_modifier = 3
    RULE_components = 4
    RULE_component = 5
    RULE_missing = 6
    RULE_degree = 7
    RULE_interval = 8
    RULE_bass = 9
    RULE_shorthand = 10

    ruleNames =  [ "chord", "root", "natural", "modifier", "components", 
                   "component", "missing", "degree", "interval", "bass", 
                   "shorthand" ]

    EOF = Token.EOF
    T__0=1
    T__1=2
    T__2=3
    T__3=4
    T__4=5
    T__5=6
    T__6=7
    T__7=8
    T__8=9
    T__9=10
    T__10=11
    T__11=12
    T__12=13
    T__13=14
    T__14=15
    T__15=16
    T__16=17
    T__17=18
    T__18=19
    T__19=20
    NATURAL=21
    FLAT=22
    SHARP=23
    I_1=24
    I_2=25
    I_3=26
    I_4=27
    I_5=28
    I_6=29
    I_7=30
    I_8=31
    I_9=32
    I_10=33
    I_11=34
    I_12=35
    I_13=36
    NO_CHORD=37
    SEPARATOR=38

    def __init__(self, input:TokenStream, output:TextIO = sys.stdout):
        super().__init__(input, output)
        self.checkVersion("4.7")
        self._interp = ParserATNSimulator(self, self.atn, self.decisionsToDFA, self.sharedContextCache)
        self._predicates = None



    class ChordContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def root(self):
            return self.getTypedRuleContext(ChordLabelParser.RootContext,0)


        def SEPARATOR(self):
            return self.getToken(ChordLabelParser.SEPARATOR, 0)

        def bass(self):
            return self.getTypedRuleContext(ChordLabelParser.BassContext,0)


        def components(self):
            return self.getTypedRuleContext(ChordLabelParser.ComponentsContext,0)


        def shorthand(self):
            return self.getTypedRuleContext(ChordLabelParser.ShorthandContext,0)


        def NO_CHORD(self):
            return self.getToken(ChordLabelParser.NO_CHORD, 0)

        def getRuleIndex(self):
            return ChordLabelParser.RULE_chord

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterChord" ):
                listener.enterChord(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitChord" ):
                listener.exitChord(self)




    def chord(self):

        localctx = ChordLabelParser.ChordContext(self, self._ctx, self.state)
        self.enterRule(localctx, 0, self.RULE_chord)
        self._la = 0 # Token type
        try:
            self.state = 37
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [ChordLabelParser.NATURAL]:
                self.enterOuterAlt(localctx, 1)
                self.state = 22
                self.root()
                self.state = 31
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==ChordLabelParser.SEPARATOR:
                    self.state = 23
                    self.match(ChordLabelParser.SEPARATOR)
                    self.state = 29
                    self._errHandler.sync(self)
                    token = self._input.LA(1)
                    if token in [ChordLabelParser.T__0]:
                        self.state = 24
                        self.components()
                        pass
                    elif token in [ChordLabelParser.T__5, ChordLabelParser.T__6, ChordLabelParser.T__7, ChordLabelParser.T__8, ChordLabelParser.T__9, ChordLabelParser.T__10, ChordLabelParser.T__11, ChordLabelParser.T__12, ChordLabelParser.T__13, ChordLabelParser.T__14, ChordLabelParser.T__15, ChordLabelParser.T__16, ChordLabelParser.T__17, ChordLabelParser.T__18, ChordLabelParser.T__19, ChordLabelParser.I_7, ChordLabelParser.I_9, ChordLabelParser.I_11, ChordLabelParser.I_13]:
                        self.state = 25
                        self.shorthand()
                        self.state = 27
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)
                        if _la==ChordLabelParser.T__0:
                            self.state = 26
                            self.components()


                        pass
                    else:
                        raise NoViableAltException(self)



                self.state = 34
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==ChordLabelParser.T__4:
                    self.state = 33
                    self.bass()


                pass
            elif token in [ChordLabelParser.NO_CHORD]:
                self.enterOuterAlt(localctx, 2)
                self.state = 36
                self.match(ChordLabelParser.NO_CHORD)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class RootContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def natural(self):
            return self.getTypedRuleContext(ChordLabelParser.NaturalContext,0)


        def modifier(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(ChordLabelParser.ModifierContext)
            else:
                return self.getTypedRuleContext(ChordLabelParser.ModifierContext,i)


        def getRuleIndex(self):
            return ChordLabelParser.RULE_root

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterRoot" ):
                listener.enterRoot(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitRoot" ):
                listener.exitRoot(self)




    def root(self):

        localctx = ChordLabelParser.RootContext(self, self._ctx, self.state)
        self.enterRule(localctx, 2, self.RULE_root)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 39
            self.natural()
            self.state = 43
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==ChordLabelParser.FLAT or _la==ChordLabelParser.SHARP:
                self.state = 40
                self.modifier()
                self.state = 45
                self._errHandler.sync(self)
                _la = self._input.LA(1)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class NaturalContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def NATURAL(self):
            return self.getToken(ChordLabelParser.NATURAL, 0)

        def getRuleIndex(self):
            return ChordLabelParser.RULE_natural

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNatural" ):
                listener.enterNatural(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNatural" ):
                listener.exitNatural(self)




    def natural(self):

        localctx = ChordLabelParser.NaturalContext(self, self._ctx, self.state)
        self.enterRule(localctx, 4, self.RULE_natural)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 46
            self.match(ChordLabelParser.NATURAL)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class ModifierContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def FLAT(self):
            return self.getToken(ChordLabelParser.FLAT, 0)

        def SHARP(self):
            return self.getToken(ChordLabelParser.SHARP, 0)

        def getRuleIndex(self):
            return ChordLabelParser.RULE_modifier

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterModifier" ):
                listener.enterModifier(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitModifier" ):
                listener.exitModifier(self)




    def modifier(self):

        localctx = ChordLabelParser.ModifierContext(self, self._ctx, self.state)
        self.enterRule(localctx, 6, self.RULE_modifier)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 48
            _la = self._input.LA(1)
            if not(_la==ChordLabelParser.FLAT or _la==ChordLabelParser.SHARP):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class ComponentsContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def component(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(ChordLabelParser.ComponentContext)
            else:
                return self.getTypedRuleContext(ChordLabelParser.ComponentContext,i)


        def getRuleIndex(self):
            return ChordLabelParser.RULE_components

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterComponents" ):
                listener.enterComponents(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitComponents" ):
                listener.exitComponents(self)




    def components(self):

        localctx = ChordLabelParser.ComponentsContext(self, self._ctx, self.state)
        self.enterRule(localctx, 8, self.RULE_components)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 50
            self.match(ChordLabelParser.T__0)
            self.state = 51
            self.component()
            self.state = 56
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==ChordLabelParser.T__1:
                self.state = 52
                self.match(ChordLabelParser.T__1)
                self.state = 53
                self.component()
                self.state = 58
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 59
            self.match(ChordLabelParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class ComponentContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def degree(self):
            return self.getTypedRuleContext(ChordLabelParser.DegreeContext,0)


        def missing(self):
            return self.getTypedRuleContext(ChordLabelParser.MissingContext,0)


        def getRuleIndex(self):
            return ChordLabelParser.RULE_component

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterComponent" ):
                listener.enterComponent(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitComponent" ):
                listener.exitComponent(self)




    def component(self):

        localctx = ChordLabelParser.ComponentContext(self, self._ctx, self.state)
        self.enterRule(localctx, 10, self.RULE_component)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 62
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==ChordLabelParser.T__3:
                self.state = 61
                self.missing()


            self.state = 64
            self.degree()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class MissingContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return ChordLabelParser.RULE_missing

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterMissing" ):
                listener.enterMissing(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitMissing" ):
                listener.exitMissing(self)




    def missing(self):

        localctx = ChordLabelParser.MissingContext(self, self._ctx, self.state)
        self.enterRule(localctx, 12, self.RULE_missing)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 66
            self.match(ChordLabelParser.T__3)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class DegreeContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def interval(self):
            return self.getTypedRuleContext(ChordLabelParser.IntervalContext,0)


        def modifier(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(ChordLabelParser.ModifierContext)
            else:
                return self.getTypedRuleContext(ChordLabelParser.ModifierContext,i)


        def getRuleIndex(self):
            return ChordLabelParser.RULE_degree

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDegree" ):
                listener.enterDegree(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDegree" ):
                listener.exitDegree(self)




    def degree(self):

        localctx = ChordLabelParser.DegreeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 14, self.RULE_degree)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 71
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==ChordLabelParser.FLAT or _la==ChordLabelParser.SHARP:
                self.state = 68
                self.modifier()
                self.state = 73
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 74
            self.interval()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class IntervalContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def I_1(self):
            return self.getToken(ChordLabelParser.I_1, 0)

        def I_2(self):
            return self.getToken(ChordLabelParser.I_2, 0)

        def I_3(self):
            return self.getToken(ChordLabelParser.I_3, 0)

        def I_4(self):
            return self.getToken(ChordLabelParser.I_4, 0)

        def I_5(self):
            return self.getToken(ChordLabelParser.I_5, 0)

        def I_6(self):
            return self.getToken(ChordLabelParser.I_6, 0)

        def I_7(self):
            return self.getToken(ChordLabelParser.I_7, 0)

        def I_8(self):
            return self.getToken(ChordLabelParser.I_8, 0)

        def I_9(self):
            return self.getToken(ChordLabelParser.I_9, 0)

        def I_10(self):
            return self.getToken(ChordLabelParser.I_10, 0)

        def I_11(self):
            return self.getToken(ChordLabelParser.I_11, 0)

        def I_12(self):
            return self.getToken(ChordLabelParser.I_12, 0)

        def I_13(self):
            return self.getToken(ChordLabelParser.I_13, 0)

        def getRuleIndex(self):
            return ChordLabelParser.RULE_interval

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterInterval" ):
                listener.enterInterval(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitInterval" ):
                listener.exitInterval(self)




    def interval(self):

        localctx = ChordLabelParser.IntervalContext(self, self._ctx, self.state)
        self.enterRule(localctx, 16, self.RULE_interval)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 76
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << ChordLabelParser.I_1) | (1 << ChordLabelParser.I_2) | (1 << ChordLabelParser.I_3) | (1 << ChordLabelParser.I_4) | (1 << ChordLabelParser.I_5) | (1 << ChordLabelParser.I_6) | (1 << ChordLabelParser.I_7) | (1 << ChordLabelParser.I_8) | (1 << ChordLabelParser.I_9) | (1 << ChordLabelParser.I_10) | (1 << ChordLabelParser.I_11) | (1 << ChordLabelParser.I_12) | (1 << ChordLabelParser.I_13))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class BassContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def degree(self):
            return self.getTypedRuleContext(ChordLabelParser.DegreeContext,0)


        def getRuleIndex(self):
            return ChordLabelParser.RULE_bass

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterBass" ):
                listener.enterBass(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitBass" ):
                listener.exitBass(self)




    def bass(self):

        localctx = ChordLabelParser.BassContext(self, self._ctx, self.state)
        self.enterRule(localctx, 18, self.RULE_bass)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 78
            self.match(ChordLabelParser.T__4)
            self.state = 79
            self.degree()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class ShorthandContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def I_7(self):
            return self.getToken(ChordLabelParser.I_7, 0)

        def I_9(self):
            return self.getToken(ChordLabelParser.I_9, 0)

        def I_11(self):
            return self.getToken(ChordLabelParser.I_11, 0)

        def I_13(self):
            return self.getToken(ChordLabelParser.I_13, 0)

        def getRuleIndex(self):
            return ChordLabelParser.RULE_shorthand

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterShorthand" ):
                listener.enterShorthand(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitShorthand" ):
                listener.exitShorthand(self)




    def shorthand(self):

        localctx = ChordLabelParser.ShorthandContext(self, self._ctx, self.state)
        self.enterRule(localctx, 20, self.RULE_shorthand)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 81
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << ChordLabelParser.T__5) | (1 << ChordLabelParser.T__6) | (1 << ChordLabelParser.T__7) | (1 << ChordLabelParser.T__8) | (1 << ChordLabelParser.T__9) | (1 << ChordLabelParser.T__10) | (1 << ChordLabelParser.T__11) | (1 << ChordLabelParser.T__12) | (1 << ChordLabelParser.T__13) | (1 << ChordLabelParser.T__14) | (1 << ChordLabelParser.T__15) | (1 << ChordLabelParser.T__16) | (1 << ChordLabelParser.T__17) | (1 << ChordLabelParser.T__18) | (1 << ChordLabelParser.T__19) | (1 << ChordLabelParser.I_7) | (1 << ChordLabelParser.I_9) | (1 << ChordLabelParser.I_11) | (1 << ChordLabelParser.I_13))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx








from antlr4 import InputStream, CommonTokenStream, ParseTreeWalker
from antlr4.error.ErrorListener import ErrorListener

from chord_labels.parser.ChordLabelLexer import ChordLabelLexer
from chord_labels.parser.ChordLabelParser import ChordLabelParser
from chord_labels.parser.ChordLabelListener import ChordLabelListener

def parse_chord(label):
    """
    Parses a string chord label from a string form to ChorlLabel instance
    (containing a set of pitch classes, root, bass).

    Examples:
    ```
    from chord_labels import parse_chord

    chord = parse_chord("C:maj7")
    assert chord.tones == [0, 4, 7, 11]
    assert chord.tones_binary == [1, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 1]

    assert parse_chord("F#").root == 6

    assert parse_chord("C#/5").bass == 8
    ```
    """
    lexer = ChordLabelLexer(InputStream(label))
    stream = CommonTokenStream(lexer)
    parser = ChordLabelParser(stream)
    parser._listeners = [ChordErrorListener()]
    chordContext = parser.chord()
    walker = ParseTreeWalker()
    listener = ChordLabelReader()
    walker.walk(listener, chordContext)
    return listener.chord_label

OCTAVE_SIZE = 12

DEFAULT_SHORTHAND = "maj"
# upper tones, except for the root
SHORTHANDS = {
    "maj": set([4, 7]),
    "min": set([3, 7]),
    "dim": set([3, 6]),
    "aug": set([4, 8]),
    "maj7": set([4, 7, 11]),
    "min7": set([3, 7, 10]),
    "dim7": set([3, 6, 9]),
    "hdim7": set([3, 6, 10]),
    "minmaj7": set([3, 7, 11]),
    "maj6": set([4, 7, 9]),
    "min6": set([3, 7, 9]),
    "maj9": set([4, 7, 11, 2]),
    "min9": set([3, 7, 10, 2]),
    "sus4": set([5, 7]),
    "sus2": set([2, 7]),
    "7": set([4, 7, 10]),
    "9": set([4, 7, 10, 2]),
    "11": set([4, 7, 10, 2, 5]),
    "13": set([4, 7, 10, 2, 5, 9]),
}
PITCH_CLASS_FROM_NAME = {
    "A": 9,
    "B": 11,
    "C": 0,
    "D": 2,
    "E": 4,
    "F": 5,
    "G": 7,
}
# diatonic scale interval -> pitch class
PITCH_CLASS_FROM_INTERVAL = {
    1: 0,
    2: 2,
    3: 4,
    4: 5,
    5: 7,
    6: 9,
    7: 11,
}
PITCH_CLASS_FROM_MODIFIER = {
    "b": OCTAVE_SIZE - 1,
    "#": 1,
}

class ChordLabel:
    # tones: set - absolute pitch classes
    # root: int - absolute pitch class
    # bass: int - absolute pitch class
    # title: str
    def __init__(self, tones, root, bass, title=""):
        tones_set = set(tones) if not isinstance(tones, set) else tones
        self.tones = sorted(tones_set)
        self.root = root
        self.bass = bass
        self.title = title
        # eg. [0, 4, 7, 11] - > [1, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 1]
        self.tones_binary = [int(i in tones) for i in range(OCTAVE_SIZE)]

    def __eq__(self, other):
        """Overrides the default implementation"""
        if isinstance(self, other.__class__):
            return self.__dict__ == other.__dict__
        return False

    def __repr__(self):
        return "ChordLabel({}, {}, {}, {})".format(
            repr(self.tones), repr(self.root), repr(self.bass), repr(self.title))

class ChordLabelReader(ChordLabelListener):

    def __init__(self):
        # Currently processed pitch class. For root tone it is absolute, for
        # components (including the bass tone) it is relative to the root.
        #
        # This variable might be modified by the diatonic natural tone symbol (for
        # root), diatonic interval symbol (for further components) and flat/sharp
        # modifiers.
        self.pitch_class = 0
        # Indicates whether the current component should be added (default) or
        # removed as marked by an asterisk (*).
        self.should_add_component = True
        self.any_component_specified = False
        self.abs_root = 0
        self.relative_tones = set()
        self.relative_bass = 0
        self.is_empty_chord = True
        # the output
        self.chord_label = None

    # ctx: ChordLabelParser.ModifierContext
    def enterModifier(self, ctx):
        self.pitch_class = self._pitch_class(self.pitch_class + PITCH_CLASS_FROM_MODIFIER[ctx.getText()])

    # ctx: ChordLabelParser.ShorthandContext
    def enterShorthand(self, ctx):
        self.relative_tones |= SHORTHANDS[ctx.getText()]
        self.any_component_specified = True

    # ctx: ChordLabelParser.RootContext
    def exitRoot(self, ctx):
        self.abs_root = self.pitch_class
        self.relative_tones.add(0)

    # ctx: ChordLabelParser.BassContext
    def enterBass(self, ctx):
        self.pitch_class = 0

    # ctx: ChordLabelParser.BassContext
    def exitBass(self, ctx):
        self.relative_bass = self.pitch_class
        self.relative_tones.add(self.relative_bass)

    # ctx: ChordLabelParser.NaturalContext
    def enterNatural(self, ctx):
        self.is_empty_chord = False
        self.pitch_class = PITCH_CLASS_FROM_NAME[ctx.getText()]

    # ctx: ComponentContext
    def enterComponent(self, ctx):
        self.pitch_class = 0
        self.should_add_component = True
        self.any_component_specified = True

    # ctx: ComponentContext
    def exitComponent(self, ctx):
        if self.should_add_component:
            self.relative_tones.add(self.pitch_class)
        else:
            self.relative_tones.remove(self.pitch_class)

    # ctx: IntervalContext
    def enterInterval(self, ctx):
        interval = int(ctx.getText())
        # normalize diatonic interval to [1; 7]
        normalized_interval = ((interval - 1) % 7) + 1
        self.pitch_class = self._pitch_class(self.pitch_class + PITCH_CLASS_FROM_INTERVAL[normalized_interval])

    # ctx: MissingContext
    def enterMissing(self, ctx):
        self.should_add_component = False

    # Resets the chord state.
    # ctx: ChordLabelParser.ChordContext
    def enterChord(self, ctx):
        self.pitch_class = 0
        self.should_add_component = True
        self.any_component_specified = False
        self.abs_root = 0
        self.relative_tones = set()
        self.relative_bass = 0
        self.is_empty_chord = True

    # ctx: ChordLabelParser.ChordContext
    def exitChord(self, ctx):
        if not self.is_empty_chord and not self.any_component_specified:
            self.relative_tones |= SHORTHANDS[DEFAULT_SHORTHAND]

        # convert all pitch classes from relative to absolute
        tones = [self._pitch_class(tone + self.abs_root) for tone in self.relative_tones]
        root = self._pitch_class(self.abs_root)
        bass = self._pitch_class(self.relative_bass + root)
        title = ctx.getText()
        self.chord_label = ChordLabel(tones, root, bass, title)

    def _pitch_class(self, pitch):
        return pitch % OCTAVE_SIZE

class ChordException(Exception):
    pass

class ChordErrorListener(ErrorListener):
    def __init__(self):
        super(ChordErrorListener, self).__init__()

    def syntaxError(self, recognizer, offendingSymbol, line, column, msg, e):
        raise ChordException("Syntax error: recognizer: {}, offendingSymbol: {}, line: {}, column: {}, msg: {}, e: {}".format(
            recognizer, offendingSymbol, line, column, msg, e))

    def reportAmbiguity(self, recognizer, dfa, startIndex, stopIndex, exact, ambigAlts, configs):
        raise ChordException("Ambiguity: recognizer: {}, dfs: {}, startIndex: {}, stopIndex: {}, ambigAlts: {}, configs: {}".format(
            recognizer, dfa, startIndex, stopIndex, exact, ambigAlts, configs))

    def reportAttemptingFullContext(self, recognizer, dfa, startIndex, stopIndex, conflictingAlts, configs):
        raise ChordException("Ambiguity: recognizer: {}, dfs: {}, startIndex: {}, stopIndex: {}, conflictingAlts: {}, configs: {}".format(
            recognizer, dfa, startIndex, stopIndex, exact, conflictingAlts, configs))

    def reportContextSensitivity(self, recognizer, dfa, startIndex, stopIndex, prediction, configs):
        raise ChordException("Ambiguity: recognizer: {}, dfa: {}, startIndex: {}, stopIndex: {}, prediction: {}, configs: {}".format(
            recognizer, dfa, startIndex, stopIndex, prediction, configss))



# Generated from ChordLabel.g4 by ANTLR 4.7
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .ChordLabelParser import ChordLabelParser
else:
    from ChordLabelParser import ChordLabelParser

# This class defines a complete listener for a parse tree produced by ChordLabelParser.
class ChordLabelListener(ParseTreeListener):

    # Enter a parse tree produced by ChordLabelParser#chord.
    def enterChord(self, ctx:ChordLabelParser.ChordContext):
        pass

    # Exit a parse tree produced by ChordLabelParser#chord.
    def exitChord(self, ctx:ChordLabelParser.ChordContext):
        pass


    # Enter a parse tree produced by ChordLabelParser#root.
    def enterRoot(self, ctx:ChordLabelParser.RootContext):
        pass

    # Exit a parse tree produced by ChordLabelParser#root.
    def exitRoot(self, ctx:ChordLabelParser.RootContext):
        pass


    # Enter a parse tree produced by ChordLabelParser#natural.
    def enterNatural(self, ctx:ChordLabelParser.NaturalContext):
        pass

    # Exit a parse tree produced by ChordLabelParser#natural.
    def exitNatural(self, ctx:ChordLabelParser.NaturalContext):
        pass


    # Enter a parse tree produced by ChordLabelParser#modifier.
    def enterModifier(self, ctx:ChordLabelParser.ModifierContext):
        pass

    # Exit a parse tree produced by ChordLabelParser#modifier.
    def exitModifier(self, ctx:ChordLabelParser.ModifierContext):
        pass


    # Enter a parse tree produced by ChordLabelParser#components.
    def enterComponents(self, ctx:ChordLabelParser.ComponentsContext):
        pass

    # Exit a parse tree produced by ChordLabelParser#components.
    def exitComponents(self, ctx:ChordLabelParser.ComponentsContext):
        pass


    # Enter a parse tree produced by ChordLabelParser#component.
    def enterComponent(self, ctx:ChordLabelParser.ComponentContext):
        pass

    # Exit a parse tree produced by ChordLabelParser#component.
    def exitComponent(self, ctx:ChordLabelParser.ComponentContext):
        pass


    # Enter a parse tree produced by ChordLabelParser#missing.
    def enterMissing(self, ctx:ChordLabelParser.MissingContext):
        pass

    # Exit a parse tree produced by ChordLabelParser#missing.
    def exitMissing(self, ctx:ChordLabelParser.MissingContext):
        pass


    # Enter a parse tree produced by ChordLabelParser#degree.
    def enterDegree(self, ctx:ChordLabelParser.DegreeContext):
        pass

    # Exit a parse tree produced by ChordLabelParser#degree.
    def exitDegree(self, ctx:ChordLabelParser.DegreeContext):
        pass


    # Enter a parse tree produced by ChordLabelParser#interval.
    def enterInterval(self, ctx:ChordLabelParser.IntervalContext):
        pass

    # Exit a parse tree produced by ChordLabelParser#interval.
    def exitInterval(self, ctx:ChordLabelParser.IntervalContext):
        pass


    # Enter a parse tree produced by ChordLabelParser#bass.
    def enterBass(self, ctx:ChordLabelParser.BassContext):
        pass

    # Exit a parse tree produced by ChordLabelParser#bass.
    def exitBass(self, ctx:ChordLabelParser.BassContext):
        pass


    # Enter a parse tree produced by ChordLabelParser#shorthand.
    def enterShorthand(self, ctx:ChordLabelParser.ShorthandContext):
        pass

    # Exit a parse tree produced by ChordLabelParser#shorthand.
    def exitShorthand(self, ctx:ChordLabelParser.ShorthandContext):
        pass





import distutils.cmd
import distutils.log
import glob
import os
from setuptools import setup
import subprocess

# Regenerating the lexer/parser code needs antlr4 installed.
# Ubuntu: $ sudo apt install antlr4
# OS X: $ brew install antlr
class AntlrGeneratorCommand(distutils.cmd.Command):
    """Runs ANTLR4 to generate lexer/parser source files."""

    description = 'run ANTLR4 to generate Python source files'
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        print('Regenerating parser code using ANTLR...')
        target_dir = 'chord_labels/parser'
        for f in glob.glob(os.path.join(target_dir, 'ChordLabel*')):
            print('rm', f)
            os.remove(f)
        command = ['antlr4', '-Dlanguage=Python3', '-o', target_dir, 'ChordLabel.g4']
        self.announce('Running command: %s' % ' '.join(command), level=distutils.log.INFO)
        subprocess.check_call(command)

setup(name='chord_labels',
      version='0.1',
      description='MIREX-style chord label parser',
      url='https://github.com/bzamecnik/chord_labels',
      author='Bohumir Zamecnik',
      author_email='bohumir.zamecnik@gmail.com',
      license='MIT',
      packages=['chord_labels', 'chord_labels.parser'],
      include_package_data=True,
      zip_safe=False,
      cmdclass={'antlr': AntlrGeneratorCommand},
      install_requires=[
        'antlr4-python3-runtime',
        'typing'
      ],
      # brew install pandoc
      # setup_requires=['setuptools-markdown'],
      # long_description_markdown_filename='README.md',
      # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
          # How mature is this project? Common values are
          #   3 - Alpha
          #   4 - Beta
          #   5 - Production/Stable
          'Development Status :: 3 - Alpha',

          'Intended Audience :: Developers',
          'Intended Audience :: Science/Research',

          'License :: OSI Approved :: MIT License',

          'Programming Language :: Python :: 3',

          'Operating System :: POSIX :: Linux'
      ],
      # entry_points={
          # 'console_scripts': [
          #     'chord_labels = chord_labels.__main__:main'
          # ]
      # }
)
